/* pam_sameuid module */

#include "config.h"

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>
#include <syslog.h>
#include <stdarg.h>
#include <pwd.h>
#include <grp.h>

#define PAM_SM_AUTH
#define PAM_SM_ACCOUNT

#include <security/pam_modules.h>
#include <security/pam_modutil.h>
#include <security/pam_ext.h>

static int	debug = 0;

PAM_EXTERN int
pam_sm_authenticate(pam_handle_t *pamh, int flags UNUSED,
		    int argc, const char **argv)
{
	const char *username = NULL;
	struct passwd *pwd;
	uid_t   from_uid, to_uid;

	for (; argc-- > 0; ++argv)
	{
		if (!strcmp(*argv, "debug"))
			debug = 1;
		else
			pam_syslog(pamh, LOG_ERR, "unknown option: %s", *argv);
	}

	if ((pam_get_user(pamh, &username, NULL) != PAM_SUCCESS) || !username)
	{
		pam_syslog(pamh, LOG_ERR, "username undefined");
		return PAM_SERVICE_ERR;
	}

	pwd = pam_modutil_getpwnam(pamh, username);
	if (!pwd)
	{
		if (debug)
			pam_syslog(pamh, LOG_WARNING,
				   "unknown user: %s", username);
		return PAM_USER_UNKNOWN;
	}

	to_uid = pwd->pw_uid;

	from_uid = getuid();
	pwd = pam_modutil_getpwuid(pamh, from_uid);
	if (!pwd)
	{
		pam_syslog(pamh, LOG_WARNING, "unknown uid: %u", from_uid);
		return PAM_SERVICE_ERR;
	}

	if (from_uid != to_uid)
	{
#if 0
		if (debug)
			pam_syslog(pamh, LOG_DEBUG,
				 "Access not yet granted to '%s' for '%s'",
				 username, pwd->pw_name);
#endif
		return PAM_IGNORE;
	} else
	{
		if (debug)
			pam_syslog(pamh, LOG_NOTICE,
				 "Access granted to '%s' for '%s'", username,
				 pwd->pw_name);
		return PAM_SUCCESS;
	}
}

PAM_EXTERN int
pam_sm_setcred(pam_handle_t *pamh UNUSED, int flags UNUSED,
	       int argc UNUSED, const char **argv UNUSED)
{
	return PAM_SUCCESS;
}

PAM_EXTERN int
pam_sm_acct_mgmt(pam_handle_t * pamh, int flags, int argc, const char **argv)
{
	return pam_sm_authenticate(pamh, flags, argc, argv);
}

#ifdef PAM_STATIC

/* static module data */

struct pam_module _pam_sameuid_modstruct = {
	"pam_sameuid",
	pam_sm_authenticate,
	pam_sm_setcred,
	pam_sm_acct_mgmt,
	NULL,
	NULL,
	NULL,
};

#endif

/*
 * Copyright (c) Dmitry V. Levin <ldv@altlinux.org>, 2000, 2005.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, and the entire permission notice in its entirety,
 *    including the disclaimer of warranties.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote
 *    products derived from this software without specific prior
 *    written permission.
 *
 * ALTERNATIVELY, this product may be distributed under the terms of
 * the GNU Public License, in which case the provisions of the GPL are
 * required INSTEAD OF the above restrictions.  (This clause is
 * necessary due to a potential bad interaction between the GPL and
 * the restrictions contained in a BSD-style copyright.)
 *
 * THIS SOFTWARE IS PROVIDED `AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 */
